"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from moovio_sdk.types import BaseModel, UNSET_SENTINEL
import pydantic
from pydantic import model_serializer
from typing import Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class FulfillmentDetailsErrorTypedDict(TypedDict):
    shipment_duration_days: NotRequired[str]
    return_policy: NotRequired[str]


class FulfillmentDetailsError(BaseModel):
    shipment_duration_days: Annotated[
        Optional[str], pydantic.Field(alias="shipmentDurationDays")
    ] = None

    return_policy: Annotated[Optional[str], pydantic.Field(alias="returnPolicy")] = None

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(["shipmentDurationDays", "returnPolicy"])
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m


try:
    FulfillmentDetailsError.model_rebuild()
except NameError:
    pass
