"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from moovio_sdk.types import BaseModel, UNSET_SENTINEL
import pydantic
from pydantic import model_serializer
from typing import Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class GeneratedByTypedDict(TypedDict):
    r"""The entity that generated the fee."""

    transfer_id: NotRequired[str]
    card_id: NotRequired[str]
    dispute_id: NotRequired[str]
    account_id: NotRequired[str]
    bank_account_id: NotRequired[str]
    invoice_id: NotRequired[str]


class GeneratedBy(BaseModel):
    r"""The entity that generated the fee."""

    transfer_id: Annotated[Optional[str], pydantic.Field(alias="transferID")] = None

    card_id: Annotated[Optional[str], pydantic.Field(alias="cardID")] = None

    dispute_id: Annotated[Optional[str], pydantic.Field(alias="disputeID")] = None

    account_id: Annotated[Optional[str], pydantic.Field(alias="accountID")] = None

    bank_account_id: Annotated[Optional[str], pydantic.Field(alias="bankAccountID")] = (
        None
    )

    invoice_id: Annotated[Optional[str], pydantic.Field(alias="invoiceID")] = None

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(
            [
                "transferID",
                "cardID",
                "disputeID",
                "accountID",
                "bankAccountID",
                "invoiceID",
            ]
        )
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m


try:
    GeneratedBy.model_rebuild()
except NameError:
    pass
