"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from moovio_sdk.types import BaseModel, UNSET_SENTINEL
import pydantic
from pydantic import model_serializer
from typing import List, Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class UpdateApplePayMerchantDomainsTypedDict(TypedDict):
    add_domains: NotRequired[List[str]]
    r"""A unique list of fully-qualified, top-level or sub-domain names to add."""
    remove_domains: NotRequired[List[str]]
    r"""A unique list of fully-qualified, top-level or sub-domain names to remove."""


class UpdateApplePayMerchantDomains(BaseModel):
    add_domains: Annotated[Optional[List[str]], pydantic.Field(alias="addDomains")] = (
        None
    )
    r"""A unique list of fully-qualified, top-level or sub-domain names to add."""

    remove_domains: Annotated[
        Optional[List[str]], pydantic.Field(alias="removeDomains")
    ] = None
    r"""A unique list of fully-qualified, top-level or sub-domain names to remove."""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(["addDomains", "removeDomains"])
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m


try:
    UpdateApplePayMerchantDomains.model_rebuild()
except NameError:
    pass
