"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .achpaymentsettings import ACHPaymentSettings, ACHPaymentSettingsTypedDict
from .cardpaymentsettings import CardPaymentSettings, CardPaymentSettingsTypedDict
from moovio_sdk.types import BaseModel, UNSET_SENTINEL
import pydantic
from pydantic import model_serializer
from typing import Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class SettingsTypedDict(TypedDict):
    r"""User provided settings to manage an account."""

    card_payment: NotRequired[CardPaymentSettingsTypedDict]
    r"""User provided settings to manage card payments. This data is only allowed on a business account."""
    ach_payment: NotRequired[ACHPaymentSettingsTypedDict]


class Settings(BaseModel):
    r"""User provided settings to manage an account."""

    card_payment: Annotated[
        Optional[CardPaymentSettings], pydantic.Field(alias="cardPayment")
    ] = None
    r"""User provided settings to manage card payments. This data is only allowed on a business account."""

    ach_payment: Annotated[
        Optional[ACHPaymentSettings], pydantic.Field(alias="achPayment")
    ] = None

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(["cardPayment", "achPayment"])
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m


try:
    Settings.model_rebuild()
except NameError:
    pass
