"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from moovio_sdk.types import BaseModel, UNSET_SENTINEL
import pydantic
from pydantic import model_serializer
from typing import Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class CardVolumeDistributionErrorTypedDict(TypedDict):
    ecommerce_percentage: NotRequired[str]
    card_present_percentage: NotRequired[str]
    mail_or_phone_percentage: NotRequired[str]
    debt_repayment_percentage: NotRequired[str]


class CardVolumeDistributionError(BaseModel):
    ecommerce_percentage: Annotated[
        Optional[str], pydantic.Field(alias="ecommercePercentage")
    ] = None

    card_present_percentage: Annotated[
        Optional[str], pydantic.Field(alias="cardPresentPercentage")
    ] = None

    mail_or_phone_percentage: Annotated[
        Optional[str], pydantic.Field(alias="mailOrPhonePercentage")
    ] = None

    debt_repayment_percentage: Annotated[
        Optional[str], pydantic.Field(alias="debtRepaymentPercentage")
    ] = None

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(
            [
                "ecommercePercentage",
                "cardPresentPercentage",
                "mailOrPhonePercentage",
                "debtRepaymentPercentage",
            ]
        )
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m


try:
    CardVolumeDistributionError.model_rebuild()
except NameError:
    pass
