"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from moovio_sdk.types import BaseModel, UNSET_SENTINEL
import pydantic
from pydantic import model_serializer
from typing import Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class AmountDecimalUpdateTypedDict(TypedDict):
    currency: NotRequired[str]
    r"""A 3-letter ISO 4217 currency code."""
    value_decimal: NotRequired[str]
    r"""A decimal-formatted numerical string that represents up to 9 decimal place precision.

    For example, $12.987654321 is '12.987654321'.
    """


class AmountDecimalUpdate(BaseModel):
    currency: Optional[str] = None
    r"""A 3-letter ISO 4217 currency code."""

    value_decimal: Annotated[Optional[str], pydantic.Field(alias="valueDecimal")] = None
    r"""A decimal-formatted numerical string that represents up to 9 decimal place precision.

    For example, $12.987654321 is '12.987654321'.
    """

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(["currency", "valueDecimal"])
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m


try:
    AmountDecimalUpdate.model_rebuild()
except NameError:
    pass
